@@ -146,10 +146,18 @@ void CyclicCoordinateDescent::setNoiseLevel(NoiseLevels noise) {
 	noiseLevel = noise;
 }
 
-string CyclicCoordinateDescent::getPriorInfo() {
+string CyclicCoordinateDescent::getPriorInfo() const {
 	return jointPrior->getDescription();
 }
 
+void CyclicCoordinateDescent::setCrossValidationInfo(string info) {
+    crossValidationInfo = info;
+}
+
+string CyclicCoordinateDescent::getCrossValidationInfo() const {
+    return crossValidationInfo;
+}
+
 void CyclicCoordinateDescent::setPrior(priors::JointPriorPtr newPrior) {
     jointPrior = newPrior;
 }
@@ -1027,7 +1035,7 @@ double CyclicCoordinateDescent::ccdUpdateBeta(int index) {
 	    gh.second = 0.0;
 	}
 
-    return jointPrior->getDelta(gh, hBeta, index);
+	return jointPrior->getDelta(gh, hBeta, index);
 }
 
 template <class IteratorType>
@@ -1114,15 +1122,22 @@ double CyclicCoordinateDescent::computeConvergenceCriterion(double newObjFxn, do
 	return abs(newObjFxn - oldObjFxn) / (abs(newObjFxn) + 1.0);
 }
 
-double CyclicCoordinateDescent::applyBounds(double inDelta, int index) {
-	double delta = inDelta;
-	if (delta < -hDelta[index]) {
-		delta = -hDelta[index];
-	} else if (delta > hDelta[index]) {
-		delta = hDelta[index];
-	}
+double CyclicCoordinateDescent::applyBounds(double delta, int index) {
+
+    auto doBound = true;
+    if (doBound) {
+	    if (delta < -hDelta[index]) {
+		    delta = -hDelta[index];
+	    } else if (delta > hDelta[index]) {
+		    delta = hDelta[index];
+	    }
+
+	    // TODO Remove magic numbers
+	    auto intermediate = std::max(std::abs(delta) * 2, hDelta[index] / 2);
+	    intermediate = std::max(intermediate, 1E-3);
+	    hDelta[index] = intermediate;
+    }
 
-	hDelta[index] = max(2.0 * abs(delta), 0.5 * hDelta[index]);
 	return delta;
 }
 
