@@ -15,16 +15,7 @@
 #include "priors/JointPrior.h"
 #include "io/ProgressLogger.h"
 
-#pragma GCC diagnostic push
-#if __GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6)
-#pragma GCC diagnostic ignored "-Wpragmas"
-#endif
-#pragma GCC diagnostic ignored "-Wunknown-pragmas"
-#pragma GCC diagnostic ignored "-Wignored-attributes" // To keep C++14 quiet
-#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
 #include <Eigen/Dense>
-#pragma GCC diagnostic pop
-
 #include <deque>
 
 #include "Types.h"
